

#undef BUILDING_AQDATABASE


#include <aqfilestorage/aqfs_file.h>
#include <aqfilestorage/aqfs_table_ptrlist.h>


#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>
#include <gwenhywfar/cgui.h>
#include <gwenhywfar/testframework.h>

#include <unistd.h>





int test_create_file_1()
{
  AQFS_FILE *fsf;
  int rv;

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }

  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_create_file_2()
{
  AQFS_FILE *fsf;
  int rv;

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - create file again (should give an error)\n");
  rv=AQFS_File_Create(fsf);
  if (rv==0) {
    fprintf(stderr, "Creating file should not have worked... \n");
    return 2;
  }

  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_open_nonexisting_file()
{
  AQFS_FILE *fsf;
  int rv;

  unlink("test.file");
  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - open file (should give an error)\n");
  rv=AQFS_File_Open(fsf, 0);
  if (rv==0) {
    fprintf(stderr, "Should not be able to open an non-existing file)\n");
    return 2;
  }

  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_open_file_readonly()
{
  AQFS_FILE *fsf;
  int rv;
  uint8_t testdata[128];

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - open file (read-only)\n");
  rv=AQFS_File_Open(fsf, 0);
  if (rv<0) {
    fprintf(stderr, "Unable to open file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - append data (should give an error)\n");
  memset(testdata, 0xa5, sizeof(testdata));
  rv=AQFS_File_AppendData(fsf, 1, testdata, sizeof(testdata));
  if (rv==0) {
    fprintf(stderr, "Should not be able to append data\n");
    return 2;
  }

  fprintf(stderr, "  - close file again\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_add_data_1()
{
  AQFS_FILE *fsf;
  int rv;
  uint8_t testdata[128];

  memset(testdata, 0xa5, sizeof(testdata));
  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - append data\n");
  rv=AQFS_File_AppendData(fsf, 1, testdata, sizeof(testdata));
  if (rv<0) {
    fprintf(stderr, "Unable to append data (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }

  AQFS_File_free(fsf);
  unlink("test.file");

  return 0;
}



int test_add_data_2()
{
  AQFS_FILE *fsf;
  int rv;
  uint8_t testdata[128];

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add first data object\n");
  memset(testdata, 0xa5, sizeof(testdata));
  rv=AQFS_File_AppendData(fsf, 1, testdata, sizeof(testdata));
  if (rv<0) {
    fprintf(stderr, "Unable to append data (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add second data object\n");
  memset(testdata, 0x5a, sizeof(testdata));
  rv=AQFS_File_AppendData(fsf, 1, testdata, sizeof(testdata));
  if (rv<0) {
    fprintf(stderr, "Unable to append data (%d)\n", rv);
    return 2;
  }


  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_read_data_1()
{
  AQFS_FILE *fsf;
  int rv;
  uint8_t testdata[128];
  uint8_t readdata[128];
  uint64_t offset;
  AQFS_ITEM_HEADER *ih;
  int64_t rd;

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add first data object\n");
  memset(testdata, 0xa5, sizeof(testdata));
  rv=AQFS_File_AppendData(fsf, 1, testdata, sizeof(testdata));
  if (rv<0) {
    fprintf(stderr, "Unable to append data (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add second data object\n");
  memset(testdata, 0x5a, sizeof(testdata));
  rv=AQFS_File_AppendData(fsf, 1, testdata, sizeof(testdata));
  if (rv<0) {
    fprintf(stderr, "Unable to append data (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - read item header\n");
  offset=AQFS_File_GetFirstItemOffset(fsf);
  ih=AQFS_ItemHeader_new();
  rv=AQFS_File_ReadItemHeaderAt(fsf, offset, ih);
  if (rv<0) {
    fprintf(stderr, "Unable to read item header (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - check item size in file\n");
  if (AQFS_ItemHeader_GetSizeUsed(ih)!=sizeof(readdata)) {
    fprintf(stderr, "Bad size of stored data (%d bytes instead of %d)\n",
            (int) AQFS_ItemHeader_GetSizeUsed(ih), (int)(sizeof(readdata)));
    return 2;
  }

  fprintf(stderr, "  - read data\n");
  rd=AQFS_File_ReadAt(fsf, AQFS_ItemHeader_GetOffsetDataStart(ih), readdata, sizeof(readdata));
  if (rd!=sizeof(readdata)) {
    fprintf(stderr, "Bad size read (%d bytes instead of %d)\n",
            (int) rd, (int)(sizeof(readdata)));
    return 2;
  }

  fprintf(stderr, "  - check data\n");
  memset(testdata, 0xa5, sizeof(testdata));
  if (memcmp(testdata, readdata, sizeof(readdata))!=0) {
    fprintf(stderr, "Bad data\n");
    GWEN_Text_DumpString((const char *) readdata, sizeof(readdata), 2);
    return 2;
  }

  AQFS_ItemHeader_free(ih);

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_read_data_2()
{
  AQFS_FILE *fsf;
  int rv;
  uint8_t testdata[128];
  uint8_t readdata[128];
  uint64_t offset;
  AQFS_ITEM_HEADER *ih;
  int64_t rd;

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add first data object\n");
  memset(testdata, 0xa5, sizeof(testdata));
  rv=AQFS_File_AppendData(fsf, 1, testdata, sizeof(testdata));
  if (rv<0) {
    fprintf(stderr, "Unable to append data (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add second data object\n");
  memset(testdata, 0x5a, sizeof(testdata));
  rv=AQFS_File_AppendData(fsf, 1, testdata, sizeof(testdata));
  if (rv<0) {
    fprintf(stderr, "Unable to append data (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - read first item header\n");
  offset=AQFS_File_GetFirstItemOffset(fsf);

  ih=AQFS_ItemHeader_new();
  rv=AQFS_File_ReadItemHeaderAt(fsf, offset, ih);
  if (rv<0) {
    fprintf(stderr, "Unable to read item header (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - seek second item\n");
  offset=AQFS_ItemHeader_GetOffsetDataStart(ih)+AQFS_ItemHeader_GetSizeAllocated(ih);
  AQFS_ItemHeader_free(ih);

  fprintf(stderr, "  - read second item header\n");
  ih=AQFS_ItemHeader_new();
  rv=AQFS_File_ReadItemHeaderAt(fsf, offset, ih);
  if (rv<0) {
    fprintf(stderr, "Unable to read item header (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - check item size in file\n");
  if (AQFS_ItemHeader_GetSizeUsed(ih)!=sizeof(readdata)) {
    fprintf(stderr, "Bad size of stored data (%d bytes instead of %d)\n",
            (int) AQFS_ItemHeader_GetSizeUsed(ih), (int)(sizeof(readdata)));
    return 2;
  }

  fprintf(stderr, "  - read data\n");
  rd=AQFS_File_ReadAt(fsf, AQFS_ItemHeader_GetOffsetDataStart(ih), readdata, sizeof(readdata));
  if (rd!=sizeof(readdata)) {
    fprintf(stderr, "Bad size read (%d bytes instead of %d)\n",
            (int) rd, (int)(sizeof(readdata)));
    return 2;
  }

  fprintf(stderr, "  - check data\n");
  memset(testdata, 0x5a, sizeof(testdata));
  if (memcmp(testdata, readdata, sizeof(readdata))!=0) {
    fprintf(stderr, "Bad data\n");
    GWEN_Text_DumpString((const char *) readdata, sizeof(readdata), 2);
    return 2;
  }

  AQFS_ItemHeader_free(ih);

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_add_table_1()
{
  AQFS_FILE *fsf;
  int rv;
  AQFS_TABLE *ft;
  int64_t rd;

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }


  fprintf(stderr, "  - append table\n");
  ft=AQFS_Table_Create(128);
  rd=AQFS_File_AppendTable(fsf, ft);
  if (rd<0) {
    fprintf(stderr, "Unable to append table (%d)\n", (int) rd);
    return 2;
  }

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }

  AQFS_File_free(fsf);
  unlink("test.file");

  return 0;
}



int test_add_items_to_table_1()
{
  AQFS_FILE *fsf;
  int rv;
  uint8_t testdata[128];
  uint8_t readdata[128];
  uint64_t offset;
  AQFS_ITEM_HEADER *ih;
  int64_t rd;
  AQFS_TABLE_LIST *ftl;
  AQFS_TABLE *ft;

  ftl=AQFS_Table_List_new();

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - append table\n");
  ft=AQFS_Table_Create(128);
  rd=AQFS_File_AppendTable(fsf, ft);
  if (rd<0) {
    fprintf(stderr, "Unable to append table (%d)\n", (int) rd);
    return 2;
  }
  AQFS_Table_free(ft);

  fprintf(stderr, " - read table list\n");
  offset=AQFS_File_GetFirstItemOffset(fsf);
  rv=AQFS_File_ReadTableList(fsf, offset, ftl);
  if (rv<0) {
    fprintf(stderr, "Unable to read table list (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add first data object\n");
  memset(testdata, 0xa5, sizeof(testdata));
  rd=AQFS_File_AppendData(fsf, AQFS_ITEMHEADER_TYPE_GENERIC_DATA, testdata, sizeof(testdata));
  if (rd<0) {
    fprintf(stderr, "Unable to append data (%d)\n", (int) rd);
    return 2;
  }

  fprintf(stderr, "  - adding first entry to table\n");
  rd=AQFS_File_AddTableEntry(fsf, ftl, rd);
  if (rd<0) {
    fprintf(stderr, "Unable to append entry (%d)\n", (int) rd);
    return 2;
  }
  fprintf(stderr, "    -> index=%ld\n", (long int) rd);

  fprintf(stderr, "  - add second data object\n");
  memset(testdata, 0x5a, sizeof(testdata));
  rd=AQFS_File_AppendData(fsf, AQFS_ITEMHEADER_TYPE_GENERIC_DATA, testdata, sizeof(testdata));
  if (rd<0) {
    fprintf(stderr, "Unable to append data (%d)\n", (int) rd);
    return 2;
  }

  fprintf(stderr, "  - adding 2nd entry to table\n");
  rd=AQFS_File_AddTableEntry(fsf, ftl, rd);
  if (rd<0) {
    fprintf(stderr, "Unable to append entry (%d)\n", (int) rd);
    return 2;
  }
  fprintf(stderr, "    -> index=%ld\n", (long int) rd);




  fprintf(stderr, "  - get position of first object\n");
  offset=AQFS_File_GetTableEntry(fsf, ftl, 0);
  if (offset<0) {
    fprintf(stderr, "Object not found (%d)\n", (int) offset);
    return 2;
  }

  fprintf(stderr, "  - read item header\n");
  ih=AQFS_ItemHeader_new();
  rv=AQFS_File_ReadItemHeaderAt(fsf, offset, ih);
  if (rv<0) {
    fprintf(stderr, "Unable to read item header (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - check item size in file\n");
  if (AQFS_ItemHeader_GetSizeUsed(ih)!=sizeof(readdata)) {
    fprintf(stderr, "Bad size of stored data (%d bytes instead of %d)\n",
            (int) AQFS_ItemHeader_GetSizeUsed(ih), (int)(sizeof(readdata)));
    return 2;
  }

  fprintf(stderr, "  - read data\n");
  rd=AQFS_File_ReadAt(fsf, AQFS_ItemHeader_GetOffsetDataStart(ih), readdata, sizeof(readdata));
  if (rd!=sizeof(readdata)) {
    fprintf(stderr, "Bad size read (%d bytes instead of %d)\n",
            (int) rd, (int)(sizeof(readdata)));
    return 2;
  }

  fprintf(stderr, "  - check data\n");
  memset(testdata, 0xa5, sizeof(testdata));
  if (memcmp(testdata, readdata, sizeof(readdata))!=0) {
    fprintf(stderr, "Bad data\n");
    GWEN_Text_DumpString((const char *) readdata, sizeof(readdata), 2);
    return 2;
  }

  AQFS_ItemHeader_free(ih);

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_add_items_to_table_2()
{
  AQFS_FILE *fsf;
  int rv;
  uint64_t offset;
  int64_t rd;
  AQFS_TABLE_LIST *ftl;
  AQFS_TABLE *ft;
  int i;

  ftl=AQFS_Table_List_new();

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - append table\n");
  ft=AQFS_Table_Create(4);
  rd=AQFS_File_AppendTable(fsf, ft);
  if (rd<0) {
    fprintf(stderr, "Unable to append table (%d)\n", (int) rd);
    return 2;
  }
  AQFS_Table_free(ft);

  fprintf(stderr, " - read table list\n");
  offset=AQFS_File_GetFirstItemOffset(fsf);
  rv=AQFS_File_ReadTableList(fsf, offset, ftl);
  if (rv<0) {
    fprintf(stderr, "Unable to read table list (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - adding entries to table\n");
  for (i=1; i<5; i++) {
    fprintf(stderr, "    - entry %d\n", i);
    rd=AQFS_File_AddTableEntry(fsf, ftl, i);
    if (rd<0) {
      fprintf(stderr, "Unable to append entry (%d)\n", (int) rd);
      return 2;
    }
    fprintf(stderr, "      -> index=%ld\n", (long int) rd);
  }

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_add_items_to_table_3()
{
  AQFS_FILE *fsf;
  int rv;
  uint64_t offset;
  int64_t rd;
  AQFS_TABLE_LIST *ftl;
  AQFS_TABLE *ft;
  int i;

  ftl=AQFS_Table_List_new();

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - append table\n");
  ft=AQFS_Table_Create(4);
  rd=AQFS_File_AppendTable(fsf, ft);
  if (rd<0) {
    fprintf(stderr, "Unable to append table (%d)\n", (int) rd);
    return 2;
  }
  AQFS_Table_free(ft);

  fprintf(stderr, " - read table list\n");
  offset=AQFS_File_GetFirstItemOffset(fsf);
  rv=AQFS_File_ReadTableList(fsf, offset, ftl);
  if (rv<0) {
    fprintf(stderr, "Unable to read table list (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - adding entries to table\n");
  for (i=1; i<6; i++) {
    fprintf(stderr, "    - entry %d\n", i);
    rd=AQFS_File_AddTableEntry(fsf, ftl, i);
    if (rd<0) {
      fprintf(stderr, "Unable to append entry (%d)\n", (int) rd);
      return 2;
    }
    fprintf(stderr, "      -> index=%ld\n", (long int) rd);
  }

  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_add_items_to_table_4()
{
  AQFS_FILE *fsf;
  int rv;
  uint8_t testdata[128];
  uint8_t readdata[128];
  uint64_t offset;
  AQFS_ITEM_HEADER *ih;
  int64_t rd;
  AQFS_TABLE_LIST *ftl;
  AQFS_TABLE *ft;
  int i;

  ftl=AQFS_Table_List_new();

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - append table\n");
  ft=AQFS_Table_Create(128);
  rd=AQFS_File_AppendTable(fsf, ft);
  if (rd<0) {
    fprintf(stderr, "Unable to append table (%d)\n", (int) rd);
    return 2;
  }
  AQFS_Table_free(ft);

  fprintf(stderr, " - read table list\n");
  offset=AQFS_File_GetFirstItemOffset(fsf);
  rv=AQFS_File_ReadTableList(fsf, offset, ftl);
  if (rv<0) {
    fprintf(stderr, "Unable to read table list (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add data objects\n");
  for (i=0; i<256; i++) {
    fprintf(stderr, "    - object %d\n", i);
    memset(testdata, i, sizeof(testdata));
    fprintf(stderr, "      - add data object\n");
    rd=AQFS_File_AppendData(fsf, AQFS_ITEMHEADER_TYPE_GENERIC_DATA, testdata, sizeof(testdata));
    if (rd<0) {
      fprintf(stderr, "Unable to append data (%d)\n", (int) rd);
      return 2;
    }

    fprintf(stderr, "      - add entry to table\n");
    rd=AQFS_File_AddTableEntry(fsf, ftl, rd);
    if (rd<0) {
      fprintf(stderr, "Unable to append entry (%d)\n", (int) rd);
      return 2;
    }
    fprintf(stderr, "        -> index=%ld\n", (long int) rd);
  }

  fprintf(stderr, "  - read data objects\n");
  for (i=0; i<256; i++) {
    fprintf(stderr, "    - object %d\n", i);
    fprintf(stderr, "      - get position of object\n");
    offset=AQFS_File_GetTableEntry(fsf, ftl, i);
    if (offset<0) {
      fprintf(stderr, "Object not found (%d)\n", (int) offset);
      return 2;
    }

    fprintf(stderr, "      - read item header\n");
    ih=AQFS_ItemHeader_new();
    rv=AQFS_File_ReadItemHeaderAt(fsf, offset, ih);
    if (rv<0) {
      fprintf(stderr, "Unable to read item header (%d)\n", rv);
      return 2;
    }

    fprintf(stderr, "      - check item size in file\n");
    if (AQFS_ItemHeader_GetSizeUsed(ih)!=sizeof(readdata)) {
      fprintf(stderr, "Bad size of stored data (%d bytes instead of %d)\n",
              (int) AQFS_ItemHeader_GetSizeUsed(ih), (int)(sizeof(readdata)));
      return 2;
    }

    fprintf(stderr, "      - read data\n");
    rd=AQFS_File_ReadAt(fsf, AQFS_ItemHeader_GetOffsetDataStart(ih), readdata, sizeof(readdata));
    if (rd!=sizeof(readdata)) {
      fprintf(stderr, "Bad size read (%d bytes instead of %d)\n",
              (int) rd, (int)(sizeof(readdata)));
      return 2;
    }

    fprintf(stderr, "      - check data\n");
    memset(testdata, i, sizeof(testdata));
    if (memcmp(testdata, readdata, sizeof(readdata))!=0) {
      fprintf(stderr, "Bad data\n");
      GWEN_Text_DumpString((const char *) readdata, sizeof(readdata), 2);
      return 2;
    }

    AQFS_ItemHeader_free(ih);
  }


  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
  unlink("test.file");
  return 0;
}



int test_add_items_to_table_5()
{
  AQFS_FILE *fsf;
  int rv;
  uint8_t testdata[128];
  uint64_t offset;
  int64_t rd;
  AQFS_TABLE_LIST *ftl;
  AQFS_TABLE *ft;
  int i;

  ftl=AQFS_Table_List_new();

  fsf=AQFS_File_new("test.file");

  fprintf(stderr, "  - create file\n");
  rv=AQFS_File_Create(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to create file (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - append table\n");
  ft=AQFS_Table_Create(1024);
  rd=AQFS_File_AppendTable(fsf, ft);
  if (rd<0) {
    fprintf(stderr, "Unable to append table (%d)\n", (int) rd);
    return 2;
  }
  AQFS_Table_free(ft);

  fprintf(stderr, " - read table list\n");
  offset=AQFS_File_GetFirstItemOffset(fsf);
  rv=AQFS_File_ReadTableList(fsf, offset, ftl);
  if (rv<0) {
    fprintf(stderr, "Unable to read table list (%d)\n", rv);
    return 2;
  }

  fprintf(stderr, "  - add data objects\n");
  for (i=0; i<256; i++) {
    fprintf(stderr, "    - object %d\n", i);
    memset(testdata, i, sizeof(testdata));
    fprintf(stderr, "      - add data object\n");
    rd=AQFS_File_AppendData(fsf, AQFS_ITEMHEADER_TYPE_GENERIC_DATA, testdata, sizeof(testdata));
    if (rd<0) {
      fprintf(stderr, "Unable to append data (%d)\n", (int) rd);
      return 2;
    }

    fprintf(stderr, "      - add entry to table\n");
    rd=AQFS_File_AddTableEntry(fsf, ftl, rd);
    if (rd<0) {
      fprintf(stderr, "Unable to append entry (%d)\n", (int) rd);
      return 2;
    }
    fprintf(stderr, "        -> index=%ld\n", (long int) rd);
  }

  fprintf(stderr, "  - read data objects\n");
  for (i=0; i<256; i++) {
    uint8_t *ptr=NULL;

    fprintf(stderr, "    - object %d\n", i);
    fprintf(stderr, "      - get position of object\n");
    offset=AQFS_File_GetTableEntry(fsf, ftl, i);
    if (offset<0) {
      fprintf(stderr, "Object not found (%d)\n", (int) offset);
      return 2;
    }

    fprintf(stderr, "      - read item\n");
    rd=AQFS_File_ReadDataAt(fsf, offset, &ptr, 0);
    if (rd<0) {
      fprintf(stderr, "Unable to read item (%d)\n", (int) rd);
      return 2;
    }

    fprintf(stderr, "      - check item size in file\n");
    if (rd!=sizeof(testdata)) {
      fprintf(stderr, "Bad size of stored data (%d bytes instead of %d)\n",
              (int) rd, (int)(sizeof(testdata)));
      return 2;
    }

    fprintf(stderr, "      - check data\n");
    memset(testdata, i, sizeof(testdata));
    if (memcmp(testdata, ptr, sizeof(testdata))!=0) {
      fprintf(stderr, "Bad data\n");
      GWEN_Text_DumpString((const char *) ptr, sizeof(testdata), 2);
      return 2;
    }
    free(ptr);
  }


  fprintf(stderr, "  - close file\n");
  rv=AQFS_File_Close(fsf);
  if (rv<0) {
    fprintf(stderr, "Unable to close file (%d)\n", rv);
    return 2;
  }


  AQFS_File_free(fsf);
//  unlink("test.file");
  return 0;
}



int test_table_dup()
{
  AQFS_TABLE *ft1;
  AQFS_TABLE *ft2;

  ft1=AQFS_Table_new();
  AQFS_Table_SetMaxEntries(ft1, 24);
  AQFS_Table_SetFreeEntries(ft1, 4);
  AQFS_Table_SetHighestEntry(ft1, 20);
  AQFS_Table_SetOffsetNextTable(ft1, 202);
  AQFS_Table_SetOffsetTableHeaderStart(ft1, 101);
  AQFS_Table_SetRuntimeFlags(ft1, 0x12345678);

  ft2=AQFS_Table_dup(ft1);

  if (AQFS_Table_GetMaxEntries(ft2)!=24) {
    fprintf(stderr, "No match: maxEntries\n");
    return 2;
  }
  if (AQFS_Table_GetFreeEntries(ft2)!=4) {
    fprintf(stderr, "No match: freeEntries\n");
    return 2;
  }
  if (AQFS_Table_GetHighestEntry(ft2)!=20) {
    fprintf(stderr, "No match: highestEntries\n");
    return 2;
  }
  if (AQFS_Table_GetOffsetNextTable(ft2)!=202) {
    fprintf(stderr, "No match: offsetNextTable\n");
    return 2;
  }
  if (AQFS_Table_GetOffsetTableHeaderStart(ft2)!=101) {
    fprintf(stderr, "No match: offsetTableHeaderStart\n");
    return 2;
  }
  if (AQFS_Table_GetRuntimeFlags(ft2)!=0x12345678) {
    fprintf(stderr, "No match: runtimeFlags\n");
    return 2;
  }

  return 0;
}



int test_modules(int argc, char **argv)
{
  GWEN_GUI *gui;
  GWEN_TEST_FRAMEWORK *tf;
  int rv;

  gui=GWEN_Gui_CGui_new();
  GWEN_Gui_SetGui(gui);

  tf=TestFramework_new();

  rv=AQFS_Table_PtrList_AddTests(TestFramework_GetModulesRoot(tf));
  if (rv<0) {
    fprintf(stderr, "Adding module failed.\n");
    return 2;
  }

  rv=TestFramework_Run(tf, argc, argv);
  if (rv) {
    fprintf(stderr, "SomeError in tests failed.\n");
    GWEN_Gui_SetGui(NULL);
    GWEN_Gui_free(gui);
    return 2;
  }
  TestFramework_free(tf);

  GWEN_Gui_SetGui(NULL);
  GWEN_Gui_free(gui);
  return 0;
}





int main(int argc, char **argv)
{
  int rv;
  const char *s;

  unlink("test.file");

  rv=GWEN_Init();
  if (rv) {
    DBG_ERROR_ERR(0, rv);
    return 2;
  }


  if (!GWEN_Logger_IsOpen(AQFS_LOGDOMAIN)) {
    GWEN_Logger_Open(AQFS_LOGDOMAIN,
                     "aqfilestorage", 0,
                     GWEN_LoggerType_Console,
                     GWEN_LoggerFacility_User);
  }

  s=getenv("AQDB_LOGLEVEL");
  if (s && *s) {
    GWEN_LOGGER_LEVEL ll;

    ll=GWEN_Logger_Name2Level(s);
    GWEN_Logger_SetLevel(AQFS_LOGDOMAIN, ll);
  }
  else
    GWEN_Logger_SetLevel(AQFS_LOGDOMAIN, GWEN_LoggerLevel_Info);


  rv=test_modules(argc, argv);
  if (rv!=0)
    return 2;
  return 0;




  fprintf(stderr, "Test: Open a non-existing file\n");
  rv=test_open_nonexisting_file();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Create a file\n");
  rv=test_create_file_1();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Create a file over an existing one\n");
  rv=test_create_file_2();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Add data to a file\n");
  rv=test_add_data_1();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Add 2 data objects to a file\n");
  rv=test_add_data_2();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Open a file read-only\n");
  rv=test_open_file_readonly();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Read and compare 1st item\n");
  rv=test_read_data_1();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Read and compare 2nd item\n");
  rv=test_read_data_2();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Add table\n");
  rv=test_add_table_1();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Add entries to table\n");
  rv=test_add_items_to_table_1();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Add test entries to table\n");
  rv=test_add_items_to_table_2();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Add test entries to table (3)\n");
  rv=test_add_items_to_table_3();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Add test entries to table (4)\n");
  rv=test_add_items_to_table_4();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: Add test entries to table (5)\n");
  rv=test_add_items_to_table_5();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");

  fprintf(stderr, "Test: duplicate table object\n");
  rv=test_table_dup();
  if (rv==0)
    fprintf(stderr, "-> Passed.\n");
  else
    fprintf(stderr, "-> Failed.\n");


  return 0;
}


