/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqDataBase (c) by 2020 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_DBFILE_L_H
#define AQDB_DBFILE_L_H


#include "file/dbfile.h"

#include "aqdatabase/backendsupport/iolayer.h"


#define AQDB_DBFILE_TABLE_FLAGS_NOCHGLOG 0x0001



/** @file dbfile_l.h
 *
 * This plugin uses a single AQFS_FILE file to store the data.
 *
 * The first object after the AQFS_FILE_HEADER is a table (@ref AQFS_TABLE) which contains offset entries to the database data.
 * The following entries of that table are used so far:
 * - index 0: database header item (see @ref AQDB_DBFILE_HEADER)
 * - index 1: types table (see @ref AQDB_DBFILE_TABLE)
 *
 */


AQDB_IOLAYER *AQDB_DbFile_GetIoLayer(const AQDB_DB *db);


int AQDB_DbFile_GetMatchingObjects(AQDB_DB *db, AQDB_ID tableId,
                                   uint32_t aflags,
                                   const char *expr,
                                   uint32_t qflags,
                                   AQDB_OBJECT_LIST *ol);

uint32_t AQDB_DbFile_GetOpenFlags(const AQDB_DB *db);
void AQDB_DbFile_SetOpenFlags(AQDB_DB *db, uint32_t i);

#endif
